@@ -1,10 +1,6 @@
 import os
 
 
-def remove_file(name):
-    if os.path.isfile(name):
-        os.remove(name)
-
 def uniquify(path):
     filename, extension = os.path.splitext(path)
     counter = 1
@@ -14,3 +10,9 @@ def uniquify(path):
         counter += 1
 
     return path
+
+def iterable_in_type_repr(type_repr):
+    if 'List' in type_repr: 
+        return True
+    else:
+        return False
